@@ -11,7 +11,10 @@
 from __future__ import absolute_import
 from bisect import bisect_left
 from collections import namedtuple
-import weakref
+try:
+    from weakref import WeakSet
+except ImportError:
+    from weakrefset import WeakSet
 
 from six.moves import map, zip
 from sortedcontainers import SortedList, SortedListWithKey
@@ -21,7 +24,7 @@ from .deterministic import Determination, Segment
 
 
 __all__ = ['Gauge', 'Momentum', 'inf', 'now_or']
-__version__ = '0.2.1'
+__version__ = '0.3.0'
 
 
 class Gauge(object):
@@ -35,13 +38,32 @@ class Gauge(object):
     #: A sorted list of momenta.  The items are :class:`Momentum` objects.
     momenta = None
 
+    #: The constant maximum value.
+    max_value = None
+
+    #: The gauge to indicate maximum value.
+    max_gauge = None
+
+    #: The constant minimum value.
+    min_value = None
+
+    #: The gauge to indicate minimum value.
+    min_gauge = None
+
+    #: A weak set of gauges that refer the gauge as a limit gauge.
+    referring_gauges = None
+
     def __init__(self, value, max, min=0, at=None):
+        self.__preinit__()
         at = now_or(at)
         self.base = (at, value)
+        self._set_limits(max, min, at=at, _incomplete=True)
+
+    def __preinit__(self):
+        """Called by :meth:`__init__` and :meth:`__setstate__`."""
+        self.referring_gauges = WeakSet()
         self.momenta = SortedListWithKey(key=lambda m: m[2])  # sort by until
-        self._set_limits(max_=max, min_=min, at=at, forget_past=False)
         self._events = SortedList()
-        self._links = set()
 
     @property
     def determination(self):
@@ -68,7 +90,7 @@ class Gauge(object):
         """
         # invalidate linked gauges together.  A linked gauge refers this gauge
         # as a limit.
-        for gauge in self.linked_gauges():
+        for gauge in self.referring_gauges:
             gauge.invalidate()
         # remove the cached determination.
         try:
@@ -76,100 +98,87 @@ class Gauge(object):
         except AttributeError:
             pass
 
-    def linked_gauges(self):
-        """Yields linked gauges.  It removes dead links during an iteration."""
-        try:
-            links = list(self._links)
-        except AttributeError:
-            pass
-        else:
-            for gauge_ref in links:
-                gauge = gauge_ref()
-                if gauge is None:
-                    self._links.remove(gauge_ref)
-                    continue
-                yield gauge
-
-    @property
-    def max(self):
-        return self._max
-
-    @max.setter
-    def max(self, max):
-        self.set_max(max)
-
-    @property
-    def min(self):
-        return self._min
-
-    @min.setter
-    def min(self, min):
-        self.set_min(min)
-
     def get_max(self, at=None):
         """Predicts the current maximum value."""
-        return self._max.get(at) if self._is_max_gauge else self._max
+        if self.max_gauge is None:
+            return self.max_value
+        else:
+            return self.max_gauge.get(at)
 
     def get_min(self, at=None):
         """Predicts the current minimum value."""
-        return self._min.get(at) if self._is_min_gauge else self._min
+        if self.min_gauge is None:
+            return self.min_value
+        else:
+            return self.min_gauge.get(at)
 
-    def _set_limits(self, max_=None, min_=None, clamp=False, at=None,
-                    forget_past=True):
-        limit_attrs = [x for x in [
-            (max_, '_max', '_is_max_gauge'),
-            (min_, '_min', '_is_min_gauge'),
-        ] if x[0] is not None]
+    #: The alias of :meth:`get_max`.
+    max = get_max
+
+    #: The alias of :meth:`get_min`.
+    min = get_min
+
+    def _set_limits(self, max_=None, min_=None, at=None, _incomplete=False):
         at = now_or(at)
         forget_until = at
-        flags = []
-        for limit, attr, flag_attr in limit_attrs:
-            try:
-                prev_limit = getattr(self, attr)
-            except AttributeError:
-                prev_limit = None
+        # _incomplete=True when __init__() calls it.
+        if not _incomplete:
+            value = self.get(at)
+            inside_since = self.determination.inside_since
+        items = [('max', max_, self.max_gauge, min),
+                 ('min', min_, self.min_gauge, max)]
+        for name, limit, prev_limit_gauge, clamp in items:
+            if limit is None:
+                continue
+            if prev_limit_gauge is not None:
+                # unlink from the previous limit gauge.
+                prev_limit_gauge.referring_gauges.discard(self)
+            if isinstance(limit, Gauge):
+                limit_gauge, limit_value = limit, limit.get(at)
+                forget_until = min(forget_until, limit_gauge.base[TIME])
             else:
-                if getattr(self, flag_attr):
-                    # unlink this gauge from the previous limiting gauge.
-                    prev_limit._links.discard(weakref.ref(self))
-            is_gauge = isinstance(limit, Gauge)
-            flags.append(is_gauge)
-            if is_gauge:
-                # link this gauge to the new limiting gauge.
-                limit._links.add(weakref.ref(self))
-                forget_until = min(forget_until, limit.base[TIME])
-        if forget_past:
-            value = self.get(at=forget_until)
-        for (limit, attr, flag_attr), flag in zip(limit_attrs, flags):
-            # set the internal attribute.
-            setattr(self, attr, limit)
-            setattr(self, flag_attr, flag)
-        self.invalidate()
-        if clamp:
-            value = self.clamp(value, at=at)
-            forget_until = at
-        if forget_past:
-            self.forget_past(value, at=forget_until)
+                limit_gauge, limit_value = None, limit
+            # set limit attrs
+            value_attr, gauge_attr = name + '_value', name + '_gauge'
+            if limit_gauge is None:
+                setattr(self, value_attr, limit_value)
+                setattr(self, gauge_attr, None)
+            else:
+                setattr(self, value_attr, None)
+                setattr(self, gauge_attr, limit_gauge)
+                limit_gauge.referring_gauges.add(self)
+            if _incomplete or inside_since is None:
+                continue
+            elif inside_since <= at:
+                value = clamp(value, limit_value)
+        if _incomplete:
+            return
+        self.forget_past(value, at=forget_until)
 
-    def set_max(self, max, clamp=False, at=None):
+    def set_max(self, max, at=None):
         """Changes the maximum.
 
         :param max: a number or gauge to set as the maximum.
-        :param clamp: limits the current value to be below the new maximum.
-                      (default: ``True``)
         :param at: the time to change.  (default: now)
         """
-        self._set_limits(max_=max, clamp=clamp, at=at)
+        self._set_limits(max_=max, at=at)
 
-    def set_min(self, min, clamp=False, at=None):
+    def set_min(self, min, at=None):
         """Changes the minimum.
 
         :param min: a number or gauge to set as the minimum.
-        :param clamp: limits the current value to be above the new minimum.
-                      (default: ``True``)
         :param at: the time to change.  (default: now)
         """
-        self._set_limits(min_=min, clamp=clamp, at=at)
+        self._set_limits(min_=min, at=at)
+
+    def set_limits(self, max=None, min=None, at=None):
+        """Changes the both of maximum and minimum at once.
+
+        :param max: a number or gauge to set as the maximum.  (optional)
+        :param min: a number or gauge to set as the minimum.  (optional)
+        :param at: the time to change.  (default: now)
+        """
+        return self._set_limits(max, min, at=at)
 
     def clamp(self, value, at=None):
         """Clamps by the limits at the given time.
@@ -406,6 +415,23 @@ class Gauge(object):
             yield time, method, momentum
         yield (+inf, None, None)
 
+    def _limit_gauge_rebased(self, limit_gauge, limit_value, at=None):
+        """Be clamped by changed limit gauge."""
+        at = max(now_or(at), self.base[TIME])
+        value = self.get(at)
+        inside_since = self.determination.inside_since
+        if inside_since is None or at != self.base[TIME] and at < inside_since:
+            pass
+        else:
+            if limit_gauge is self.max_gauge:
+                clamp = min
+            elif limit_gauge is self.min_gauge:
+                clamp = max
+            else:
+                raise ValueError('The limit is neither max nor min')
+            value = clamp(value, limit_value)
+        self.forget_past(value, at=at)
+
     def _rebase(self, value=None, at=None, remove_momenta_before=None):
         """Sets the base and removes momenta between indexes of ``start`` and
         ``stop``.
@@ -419,6 +445,8 @@ class Gauge(object):
         at = now_or(at)
         if value is None:
             value = self.get(at=at)
+        for gauge in self.referring_gauges:
+            gauge._limit_gauge_rebased(self, value, at=at)
         self.base = (at, value)
         del self.momenta[:remove_momenta_before]
         self.invalidate()
@@ -440,22 +468,21 @@ class Gauge(object):
         :param at: the time base.  (default: now)
         """
         at = now_or(at)
-        for gauge in self.linked_gauges():
-            if gauge.base[TIME] < at:
-                gauge.forget_past(at=at)
         x = self.momenta.bisect_left((-inf, -inf, at))
         value = self._rebase(value, at=at, remove_momenta_before=x)
         return value
 
     def __getstate__(self):
-        limits = (self._max, self._min)
-        momenta = list(map(tuple, self.momenta))
-        return (self.base, limits, momenta)
+        return (self.base, list(map(tuple, self.momenta)),
+                self.max_value, self.max_gauge,
+                self.min_value, self.min_gauge)
 
     def __setstate__(self, state):
-        base, limits, momenta = state
-        max_, min_ = limits
-        self.__init__(base[VALUE], max=max_, min=min_, at=base[TIME])
+        base, momenta, max_value, max_gauge, min_value, min_gauge = state
+        self.__preinit__()
+        self.base = base
+        self.max_value, self.max_gauge = max_value, max_gauge
+        self.min_value, self.min_gauge = min_value, min_gauge
         for momentum in momenta:
             self.add_momentum(*momentum)
 
@@ -471,14 +498,16 @@ class Gauge(object):
         value = self.get(at=at)
         hyper = False
         limit_reprs = []
-        for limit in [self.max, self.min]:
-            if isinstance(limit, Gauge):
-                hyper = True
-                limit_reprs.append('{0!r}'.format(limit))
+        limit_items = [(self.max_value, self.max_gauge),
+                       (self.min_value, self.min_gauge)]
+        for limit_value, limit_gauge in limit_items:
+            if limit_gauge is None:
+                limit_reprs.append('{0:.2f}'.format(limit_value))
             else:
-                limit_reprs.append('{0:.2f}'.format(limit))
+                hyper = True
+                limit_reprs.append('{0!r}'.format(limit_gauge))
         form = '<{0} {1:.2f}'
-        if not hyper and self.min == 0:
+        if not hyper and self.min_value == 0:
             form += '/{2}>'
         else:
             form += ' between {3}~{2}>'
